package SDTMods
{
	import flash.events.Event;
	import flash.display.MovieClip;
	import flash.display.DisplayObjectContainer;
	import flash.media.Sound;
	import flash.utils.getDefinitionByName;
	
	public class ModPackage extends MovieClip
	{
		public const MOD_VERSION:String = "SDTMOD_0.2";
		
		public var modType:String = ModTypes.MOD_PACKAGE;
		
		private var _mods:Array = new Array();
		public var modName:String = "";
		public var modCreator:String = "anon";
		public var charData:String;
		
		public var dialogueName:String = "";
		public var clearAllDialogue:Boolean = false;
		
		public var targetElement:String;
		public var overwrite:Boolean;
		
		private var warningsGenerated:Boolean = false;
	
		public function ModPackage():void
		{
			modType = ModTypes.MOD_PACKAGE;
			var splash:SplashScreen = new SplashScreen();
			addChild(splash);
			
			addEventListener(Event.ENTER_FRAME, onFirstFrame);
		}
		
		private function onFirstFrame(e:Event):void
		{
			removeEventListener(Event.ENTER_FRAME, onFirstFrame);
			checkMod();
		}
		
		public function registerMod(newMod:*):void
		{
			_mods.push(newMod);
		}
		
		public function registerDialogue(lineType:String, dialogueLine:String, soundName:String = null, vol:Number = 0.8):void
		{
			var audio:Sound;
			var audioOkay:Boolean = false;
			if (soundName)
			{
				var SoundClass:Class;
				try
				{
					SoundClass = getDefinitionByName(soundName) as Class;
					audio = new SoundClass();
					if (audio is Sound)
					{
						audioOkay = true;
					}
					else
					{
						warn(soundName + " is not an audio file.");
					}
				}
				catch (e:Error)
				{
					warn("Could not found an audio file called "+soundName);
				}
			}
			if (audioOkay) {
				_mods.push(new DialogueMod(lineType, dialogueLine, audio, vol));
			} else {
				_mods.push(new DialogueMod(lineType, dialogueLine));
			}
		}
		
		public function setupDialogue(dialogueLibraryName:String, clearAll:Boolean = true):void
		{
			dialogueName = dialogueLibraryName;
			clearAllDialogue = clearAll;
		}
		
		public function get mods():Array
		{
			return _mods;
		}
		
		public function checkMod():void
		{
			trace ("Checking mod...");
			var modsLength:uint = _mods.length;
			for (var modID:uint = 0; modID < modsLength; modID++)
			{
				var modNo:uint = modID + 1;
				var modName:String = _mods[modID].name;
				var modRef:String = modNo + " - (" + modName + ")";
				if (_mods[modID] is DisplayObjectContainer)
				{
					if (_mods[modID].numChildren == 0)
					{
						warn ("Mod " + modRef + " is empty.");
					}
					else
					{
						if (_mods[modID].modType == ModTypes.DYNAMIC_HAIR)
						{
							trace ("Registered mod: "+_mods[modID].modType+" with "+countSegments(_mods[modID], modRef)+" segments");
						}
						else
						{
							trace ("Registered mod: "+_mods[modID].modType);
						}
					}
				}
				else if (_mods[modID] is Sound)
				{
					trace ("Registered audio mod");
				}
				else if (_mods[modID] is DialogueMod)
				{
					trace ("Registered dialogue mod" + (_mods[modID].audio? " with audio" : "") + ": " + _mods[modID].dialogueLine);
				}
				else if (_mods[modID] is String)
				{
					warn ("Mod " + modRef + " seems to be a String. Make sure there are no quote marks in your registerMod call.");
				}
				else
				{
					warn ("Mod " + modRef + " is not a DisplayObjectContainer. Make sure there are no typos in your registerMod call, and that you're registering a named mod that is on the main timeline.");
				}
			}
			
			if (modsLength == 0)
			{
				warn ("No mods registered. Make sure you've registered your mods on the main timeline Settings layer.");
			}
			else
			{
				var plural:String = _mods.length == 1? "" : "s";
				trace (_mods.length + " mod" + plural + " registered");
			}
			
			if (!warningsGenerated)
			{
				trace ("Mod checks okay!");
			}
		}
		
		private function warn(warningMessage:String):void
		{
			trace ("WARNING: " + warningMessage);
			warningsGenerated = true;
		}
		
		private function countSegments(displayObject:DisplayObjectContainer, modRef:String):uint
		{
			var looseShapes:Boolean = false;
			var numSegments:uint = 0;
			var children:uint = displayObject.numChildren;
			for (var childDepth:uint = 0; childDepth < children; childDepth++)
			{
				var nextSegment:* = displayObject.getChildAt(childDepth);
				if (nextSegment is DisplayObjectContainer)
				{
					numSegments++;
				}
				else
				{
					looseShapes = true;
				}
			}
			if (looseShapes)
			{
				warn ("Loose shapes found in dynamic hair mod "+modRef);
			}
			return numSegments;
		}
	}
	
}

import flash.media.Sound;
import SDTMods.ModTypes;

class DialogueMod
{

	private var _modType:String = ModTypes.DIALOGUE;
	private var _lineType:String;
	private var _dialogueLine:String;
	private var _audio:Sound;
	private var _volume:Number;
	
	public function DialogueMod(lineType:String, dialogueLine:String, audio:Sound = null, vol:Number = 0.8):void
	{
		_lineType = lineType;
		_dialogueLine = dialogueLine;
		_audio = audio;
		_volume = Math.max(0, Math.min(1, vol));
	}
	
	public function get name():String
	{
		return _dialogueLine;
	}
	
	public function get modType():String
	{
		return _modType;
	}
	
	public function get lineType():String
	{
		return _lineType;
	}
	
	public function get dialogueLine():String
	{
		return _dialogueLine;
	}
	
	public function get audio():Sound
	{
		return _audio;
	}
	
	public function get volume():Number
	{
		return _volume;
	}

}